@@ -16,12 +16,9 @@
 
 use std::fmt::Debug;
 
-use hyper::{Client, Url};
-use hyper::header::Headers;
-use hyper::method::Method;
+use reqwest::{Method, RequestBuilder, Url};
 
 use super::super::ApiResult;
-use super::super::service::RequestBuilder;
 
 
 /// Trait for an authentication method.
@@ -38,18 +35,15 @@ pub trait AuthMethod: BoxedClone + Debug {
         String::from("public")
     }
 
-    /// Default region to use with this authentication (if any).
-    fn default_region(&self) -> Option<String> { None }
+    /// Region used with this authentication (if any).
+    fn get_region(&self) -> Option<String> { None }
 
     /// Get a URL for the requested service.
-    fn get_endpoint(&self, client: &Client,
-                    service_type: String,
-                    endpoint_interface: Option<String>,
-                    region: Option<String>) -> ApiResult<Url>;
+    fn get_endpoint(&self, service_type: String,
+                    endpoint_interface: Option<String>) -> ApiResult<Url>;
 
     /// Create an authenticated request.
-    fn request<'a>(&self, client: &'a Client, method: Method, url: Url,
-                   headers: Headers) -> ApiResult<RequestBuilder<'a>>;
+    fn request(&self, method: Method, url: Url) -> ApiResult<RequestBuilder>;
 }
 
 
