@@ -14,7 +14,7 @@
 
 //! Low-level code to work with the service catalog.
 
-use hyper::Url;
+use reqwest::Url;
 
 use super::super::{ApiError, ApiResult};
 use super::super::utils;
@@ -23,28 +23,33 @@ use super::protocol::{CatalogRecord, Endpoint};
 
 /// Build a catalog URI.
 pub fn get_url(auth_url: Url) -> Url {
-    utils::url::extend(auth_url, &["v3", "auth", "catalog"])
+    // TODO: more robust v3 detection?
+    if auth_url.path().ends_with("/v3") {
+        utils::url::extend(auth_url, &["auth", "catalog"])
+    } else {
+        utils::url::extend(auth_url, &["v3", "auth", "catalog"])
+    }
 }
 
 /// Find an endpoint in the service catalog.
-pub fn find_endpoint(catalog: &Vec<CatalogRecord>, service_type: String,
-                     endpoint_interface: String, region: Option<String>)
-        -> ApiResult<&Endpoint> {
-    let svc = match catalog.iter().find(|x| x.service_type == service_type) {
+pub fn find_endpoint<'c>(catalog: &'c Vec<CatalogRecord>, service_type: &String,
+                     endpoint_interface: &String, region: &Option<String>)
+        -> ApiResult<&'c Endpoint> {
+    let svc = match catalog.iter().find(|x| x.service_type == *service_type) {
         Some(s) => s,
-        None => return Err(ApiError::EndpointNotFound(service_type))
+        None => return Err(ApiError::EndpointNotFound(service_type.clone()))
     };
 
     let maybe_endp: Option<&Endpoint>;
-    if let Some(rgn) = region {
+    if let Some(ref rgn) = *region {
         maybe_endp = svc.endpoints.iter().find(
-            |x| x.interface == endpoint_interface && x.region == rgn);
+            |x| x.interface == *endpoint_interface && x.region == *rgn);
     } else {
         maybe_endp = svc.endpoints.iter().find(
-            |x| x.interface == endpoint_interface);
+            |x| x.interface == *endpoint_interface);
     }
 
-    maybe_endp.ok_or(ApiError::EndpointNotFound(service_type))
+    maybe_endp.ok_or_else(|| ApiError::EndpointNotFound(service_type.clone()))
 }
 
 
@@ -101,9 +106,9 @@ pub mod test {
     fn find_endpoint<'a>(cat: &'a Vec<CatalogRecord>,
                          service_type: &str, interface_type: &str,
                          region: Option<&str>) -> ApiResult<&'a Endpoint> {
-        super::find_endpoint(cat, String::from(service_type),
-                             String::from(interface_type),
-                             region.map(String::from))
+        super::find_endpoint(cat, &String::from(service_type),
+                             &String::from(interface_type),
+                             &region.map(String::from))
     }
 
     #[test]
