@@ -53,6 +53,7 @@
         unused,
         unused_allocation,
         unused_comparisons,
+        unused_doc_comment,
         unused_extern_crates,
         unused_import_braces,
         unused_parens,
@@ -63,19 +64,14 @@
 #[cfg(feature = "compute")]
 extern crate chrono;
 #[macro_use]
-extern crate hyper;
-#[cfg(feature = "tls")]
-extern crate hyper_rustls;
-#[macro_use]
 extern crate log;
+extern crate reqwest;
 extern crate serde;
 #[macro_use]
 extern crate serde_derive;
+#[allow(unused_extern_crates)]
 extern crate serde_json;
 
-#[cfg(test)] #[macro_use]
-extern crate yup_hyper_mock;
-
 pub mod auth;
 mod common;
 #[cfg(feature = "compute")]
